#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// 
    /// </summary>
    [Description("")]
    public class SessionColors : Indicator
    {
        #region Variables
		private Color colorTokyo 	= Color.Cyan;
		private int iTokyoBegin 	= 1;
		private int iTokyoEnd 		= 10;
		private int iTokyoOpacity 	= 5;
		
		private Color colorLondon 	= Color.Yellow;
		private int iLondonBegin 	= 9;
		private int iLondonEnd 		= 18;
		private int iLondonOpacity 	= 5;
		
		private Color colorNewYork 	= Color.Fuchsia;
		private int iNewYorkBegin 	= 14;
		private int iNewYorkEnd 	= 23;
		private int iNewYorkOpacity = 5;
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
            Overlay	= true;
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
			int year 	= Convert.ToInt16(Time[0].ToString("yyyy"));
			int month	= Convert.ToInt16(Time[0].ToString("MM"));
			int day 	= Convert.ToInt16(Time[0].ToString("dd"));
			
			if (ToTime(Time[0]) >= iTokyoBegin*10000 && ToTime(Time[0]) <= iTokyoEnd*10000) {
				int barsAgo = CurrentBar - Bars.GetBar(new DateTime(year, month, day, iTokyoBegin, 0, 0));
				DrawRectangle("Tokyo_" + Time[0].ToString("MM/dd/yyyy"), false, barsAgo, Highs[0][HighestBar(High, barsAgo)], 0, Lows[0][LowestBar(Low, barsAgo)], Color.Transparent, colorTokyo, iTokyoOpacity);
			}
			if (ToTime(Time[0]) >= iLondonBegin*10000 && ToTime(Time[0]) <= iLondonEnd*10000) {
				int barsAgo = CurrentBar - Bars.GetBar(new DateTime(year, month, day, iLondonBegin, 0, 0));
				DrawRectangle("London" + Time[0].ToString("MM/dd/yyyy"), false, barsAgo, Highs[0][HighestBar(High, barsAgo)], 0, Lows[0][LowestBar(Low, barsAgo)], Color.Transparent, colorLondon, iLondonOpacity);
			}
			if (ToTime(Time[0]) >= iNewYorkBegin*10000 && ToTime(Time[0]) <= iNewYorkEnd*10000) {
				int barsAgo = CurrentBar - Bars.GetBar(new DateTime(year, month, day, iNewYorkBegin, 0, 0));
				DrawRectangle("NewYork" + Time[0].ToString("MM/dd/yyyy"), false, barsAgo, Highs[0][HighestBar(High, barsAgo)], 0, Lows[0][LowestBar(Low, barsAgo)], Color.Transparent, colorNewYork, iNewYorkOpacity);
			}
        }

        #region Properties
		[XmlIgnore()]
        [Description("Color for painted region")]
        [GridCategory("Asia Session")]
        public Color ColorTokyo
        {
            get { return colorTokyo; }
            set { colorTokyo = value; }
        }
		[Browsable(false)]
		public string ColorTokyoSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(colorTokyo); }
			set { colorTokyo = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
        [Description("Begin (hour) for painted region in local time")]
        [GridCategory("Asia Session")]
        public int TokyoBegin
        {
            get { return iTokyoBegin; }
            set { iTokyoBegin = Math.Min(Math.Max(0, value),24); }
        }		
        [Description("End (hour) for painted region in local time")]
        [GridCategory("Asia Session")]
        public int TokyoEnd
        {
            get { return iTokyoEnd; }
            set { iTokyoEnd = Math.Min(Math.Max(0, value),24); }
        }
        [Description("Opacity for painted region")]
        [GridCategory("Asia Session")]
        public int TokyoOpacity
        {
            get { return iTokyoOpacity; }
            set { iTokyoOpacity = Math.Min(Math.Max(0, value),10); }
        }
		[XmlIgnore()]
        [Description("Color for painted region")]
        [GridCategory("London Session")]
        public Color ColorLondon
        {
            get { return colorLondon; }
            set { colorLondon = value; }
        }
		[Browsable(false)]
		public string ColorLondonSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(colorLondon); }
			set { colorLondon = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
        [Description("Begin (hour) for painted region in local time")]
        [GridCategory("London Session")]
        public int LondonBegin
        {
            get { return iLondonBegin; }
            set { iLondonBegin = Math.Min(Math.Max(0, value),24); }
        }		
        [Description("End (hour) for painted region in local time")]
        [GridCategory("London Session")]
        public int LondonEnd
        {
            get { return iLondonEnd; }
            set { iLondonEnd = Math.Min(Math.Max(0, value),24); }
        }
        [Description("Opacity for painted region")]
        [GridCategory("London Session")]
        public int LondonOpacity
        {
            get { return iLondonOpacity; }
            set { iLondonOpacity = Math.Min(Math.Max(0, value),10); }
        }		
		[XmlIgnore()]
        [Description("Color for painted region")]
        [GridCategory("NewYork Session")]
        public Color ColorNewYork
        {
            get { return colorNewYork; }
            set { colorNewYork = value; }
        }
		[Browsable(false)]
		public string ColorNewYorkSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(colorNewYork); }
			set { colorNewYork = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}	
        [Description("Begin (hour) for painted region in local time")]
        [GridCategory("NewYork Session")]
        public int NewYorkBegin
        {
            get { return iNewYorkBegin; }
            set { iNewYorkBegin = Math.Min(Math.Max(0, value),24); }
        }		
        [Description("End (hour) for painted region in local time")]
        [GridCategory("NewYork Session")]
        public int NewYorkEnd
        {
            get { return iNewYorkEnd; }
            set { iNewYorkEnd = Math.Min(Math.Max(0, value),24); }
        }
        [Description("Opacity for painted region")]
        [GridCategory("NewYork Session")]
        public int NewYorkOpacity
        {
            get { return iNewYorkOpacity; }
            set { iNewYorkOpacity = Math.Min(Math.Max(0, value),10); }
        }			
		#endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private SessionColors[] cacheSessionColors = null;

        private static SessionColors checkSessionColors = new SessionColors();

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        public SessionColors SessionColors(Color colorLondon, Color colorNewYork, Color colorTokyo, int londonBegin, int londonEnd, int londonOpacity, int newYorkBegin, int newYorkEnd, int newYorkOpacity, int tokyoBegin, int tokyoEnd, int tokyoOpacity)
        {
            return SessionColors(Input, colorLondon, colorNewYork, colorTokyo, londonBegin, londonEnd, londonOpacity, newYorkBegin, newYorkEnd, newYorkOpacity, tokyoBegin, tokyoEnd, tokyoOpacity);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        public SessionColors SessionColors(Data.IDataSeries input, Color colorLondon, Color colorNewYork, Color colorTokyo, int londonBegin, int londonEnd, int londonOpacity, int newYorkBegin, int newYorkEnd, int newYorkOpacity, int tokyoBegin, int tokyoEnd, int tokyoOpacity)
        {
            if (cacheSessionColors != null)
                for (int idx = 0; idx < cacheSessionColors.Length; idx++)
                    if (cacheSessionColors[idx].ColorLondon == colorLondon && cacheSessionColors[idx].ColorNewYork == colorNewYork && cacheSessionColors[idx].ColorTokyo == colorTokyo && cacheSessionColors[idx].LondonBegin == londonBegin && cacheSessionColors[idx].LondonEnd == londonEnd && cacheSessionColors[idx].LondonOpacity == londonOpacity && cacheSessionColors[idx].NewYorkBegin == newYorkBegin && cacheSessionColors[idx].NewYorkEnd == newYorkEnd && cacheSessionColors[idx].NewYorkOpacity == newYorkOpacity && cacheSessionColors[idx].TokyoBegin == tokyoBegin && cacheSessionColors[idx].TokyoEnd == tokyoEnd && cacheSessionColors[idx].TokyoOpacity == tokyoOpacity && cacheSessionColors[idx].EqualsInput(input))
                        return cacheSessionColors[idx];

            lock (checkSessionColors)
            {
                checkSessionColors.ColorLondon = colorLondon;
                colorLondon = checkSessionColors.ColorLondon;
                checkSessionColors.ColorNewYork = colorNewYork;
                colorNewYork = checkSessionColors.ColorNewYork;
                checkSessionColors.ColorTokyo = colorTokyo;
                colorTokyo = checkSessionColors.ColorTokyo;
                checkSessionColors.LondonBegin = londonBegin;
                londonBegin = checkSessionColors.LondonBegin;
                checkSessionColors.LondonEnd = londonEnd;
                londonEnd = checkSessionColors.LondonEnd;
                checkSessionColors.LondonOpacity = londonOpacity;
                londonOpacity = checkSessionColors.LondonOpacity;
                checkSessionColors.NewYorkBegin = newYorkBegin;
                newYorkBegin = checkSessionColors.NewYorkBegin;
                checkSessionColors.NewYorkEnd = newYorkEnd;
                newYorkEnd = checkSessionColors.NewYorkEnd;
                checkSessionColors.NewYorkOpacity = newYorkOpacity;
                newYorkOpacity = checkSessionColors.NewYorkOpacity;
                checkSessionColors.TokyoBegin = tokyoBegin;
                tokyoBegin = checkSessionColors.TokyoBegin;
                checkSessionColors.TokyoEnd = tokyoEnd;
                tokyoEnd = checkSessionColors.TokyoEnd;
                checkSessionColors.TokyoOpacity = tokyoOpacity;
                tokyoOpacity = checkSessionColors.TokyoOpacity;

                if (cacheSessionColors != null)
                    for (int idx = 0; idx < cacheSessionColors.Length; idx++)
                        if (cacheSessionColors[idx].ColorLondon == colorLondon && cacheSessionColors[idx].ColorNewYork == colorNewYork && cacheSessionColors[idx].ColorTokyo == colorTokyo && cacheSessionColors[idx].LondonBegin == londonBegin && cacheSessionColors[idx].LondonEnd == londonEnd && cacheSessionColors[idx].LondonOpacity == londonOpacity && cacheSessionColors[idx].NewYorkBegin == newYorkBegin && cacheSessionColors[idx].NewYorkEnd == newYorkEnd && cacheSessionColors[idx].NewYorkOpacity == newYorkOpacity && cacheSessionColors[idx].TokyoBegin == tokyoBegin && cacheSessionColors[idx].TokyoEnd == tokyoEnd && cacheSessionColors[idx].TokyoOpacity == tokyoOpacity && cacheSessionColors[idx].EqualsInput(input))
                            return cacheSessionColors[idx];

                SessionColors indicator = new SessionColors();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                indicator.ColorLondon = colorLondon;
                indicator.ColorNewYork = colorNewYork;
                indicator.ColorTokyo = colorTokyo;
                indicator.LondonBegin = londonBegin;
                indicator.LondonEnd = londonEnd;
                indicator.LondonOpacity = londonOpacity;
                indicator.NewYorkBegin = newYorkBegin;
                indicator.NewYorkEnd = newYorkEnd;
                indicator.NewYorkOpacity = newYorkOpacity;
                indicator.TokyoBegin = tokyoBegin;
                indicator.TokyoEnd = tokyoEnd;
                indicator.TokyoOpacity = tokyoOpacity;
                Indicators.Add(indicator);
                indicator.SetUp();

                SessionColors[] tmp = new SessionColors[cacheSessionColors == null ? 1 : cacheSessionColors.Length + 1];
                if (cacheSessionColors != null)
                    cacheSessionColors.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cacheSessionColors = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.SessionColors SessionColors(Color colorLondon, Color colorNewYork, Color colorTokyo, int londonBegin, int londonEnd, int londonOpacity, int newYorkBegin, int newYorkEnd, int newYorkOpacity, int tokyoBegin, int tokyoEnd, int tokyoOpacity)
        {
            return _indicator.SessionColors(Input, colorLondon, colorNewYork, colorTokyo, londonBegin, londonEnd, londonOpacity, newYorkBegin, newYorkEnd, newYorkOpacity, tokyoBegin, tokyoEnd, tokyoOpacity);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        public Indicator.SessionColors SessionColors(Data.IDataSeries input, Color colorLondon, Color colorNewYork, Color colorTokyo, int londonBegin, int londonEnd, int londonOpacity, int newYorkBegin, int newYorkEnd, int newYorkOpacity, int tokyoBegin, int tokyoEnd, int tokyoOpacity)
        {
            return _indicator.SessionColors(input, colorLondon, colorNewYork, colorTokyo, londonBegin, londonEnd, londonOpacity, newYorkBegin, newYorkEnd, newYorkOpacity, tokyoBegin, tokyoEnd, tokyoOpacity);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.SessionColors SessionColors(Color colorLondon, Color colorNewYork, Color colorTokyo, int londonBegin, int londonEnd, int londonOpacity, int newYorkBegin, int newYorkEnd, int newYorkOpacity, int tokyoBegin, int tokyoEnd, int tokyoOpacity)
        {
            return _indicator.SessionColors(Input, colorLondon, colorNewYork, colorTokyo, londonBegin, londonEnd, londonOpacity, newYorkBegin, newYorkEnd, newYorkOpacity, tokyoBegin, tokyoEnd, tokyoOpacity);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        public Indicator.SessionColors SessionColors(Data.IDataSeries input, Color colorLondon, Color colorNewYork, Color colorTokyo, int londonBegin, int londonEnd, int londonOpacity, int newYorkBegin, int newYorkEnd, int newYorkOpacity, int tokyoBegin, int tokyoEnd, int tokyoOpacity)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.SessionColors(input, colorLondon, colorNewYork, colorTokyo, londonBegin, londonEnd, londonOpacity, newYorkBegin, newYorkEnd, newYorkOpacity, tokyoBegin, tokyoEnd, tokyoOpacity);
        }
    }
}
#endregion
